/*
 * Copyright 2011 Kai Rhr 
 *    
 *
 *    This file is part of mIDas.
 *
 *    mIDas is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    mIDas is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with mIDas.  If not, see <http://www.gnu.org/licenses/>.
 */

package pfaeff;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

import nbt.Tag;
import nbt.TagByteArray;
import nbt.TagInputStream;
import nbt.TagList;
import nbt.TagOutputStream;
import nbt.TagShort;
import region.RegionFile;

/*
 * TODO: Clean up, isolate visualization from logic and data
 */
public class IDChanger extends JFrame implements ActionListener {	
	private ArrayList<File> saveGames = new ArrayList<File>();
	private ArrayList<String> idNames = new ArrayList<String>();
	
	// Gui Elements
	private JComboBox cb_selectSaveGame;
	private JComboBox cb_selectSourceID;
	private JComboBox cb_selectTargetID;
	
	private JCheckBox c_backup;
	
	DefaultListModel model = new DefaultListModel();	
	private JList li_ID;
	
	private JLabel lb_file;
	private JLabel lb_chunk;
	private JProgressBar pb_file;
	private JProgressBar pb_chunk;
	
	public IDChanger(String title) {
		super(title);
		
		// Init Data
		initSaveGames();
		initIDNames();
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Add GUI elements
		getContentPane().add(createOpenSaveGamePanel(), BorderLayout.PAGE_START);
		getContentPane().add(createChooseIDsPanel(), BorderLayout.LINE_START);
		getContentPane().add(createProgressPanel(), BorderLayout.LINE_END);
		pack();
		
		setLocationRelativeTo(null);
		setResizable(false);		
		setVisible(true);
	}

	private void initSaveGames() {
		File mcSavePath = new File(getMCSavePath());
		saveGames = FileTools.getSubDirectories(mcSavePath);			
	}
	
	private void initIDNames() {
		File f = new File("IDNames.txt");		
		if (f.exists()) {
			try {
				idNames = readFile(f);
			} catch (IOException e) {
			}		
		}		
	}
	
	private JPanel createProgressPanel() {
		JPanel pnl_progress = new JPanel();
		pnl_progress.setLayout(new BoxLayout(pnl_progress, BoxLayout.PAGE_AXIS));
		pnl_progress.setBorder(BorderFactory.createTitledBorder("Progress"));
		pnl_progress.add(createStartPanel());
		pnl_progress.add(Box.createVerticalStrut(10));
		lb_file = new JLabel("Current File:");
		pnl_progress.add(lb_file);
		pnl_progress.add(createFileProgressBar());
		lb_chunk = new JLabel("Current Chunk:");
		pnl_progress.add(lb_chunk);		
		pnl_progress.add(createChunkProgressBar());
		
		pnl_progress.setPreferredSize(new Dimension(400, pnl_progress.getHeight()));
		
		return pnl_progress;
	}	
	
	private JPanel createStartPanel() {
		JPanel pnl_start = new JPanel(new FlowLayout());
		pnl_start.add(createStartButton());
		pnl_start.add(createBackupCheckBox());
		return pnl_start;
	}

	private JCheckBox createBackupCheckBox() {
		c_backup = new JCheckBox("Make a backup", true);
		return c_backup;
	}

	private JProgressBar createFileProgressBar() {
		pb_file = new JProgressBar(0, 100);
		pb_file.setValue(0);
		pb_file.setStringPainted(true);
		return pb_file;
	}

	private JProgressBar createChunkProgressBar() {
		pb_chunk = new JProgressBar(0, 100);
		pb_chunk.setValue(0);
		pb_chunk.setStringPainted(true);		
		return pb_chunk;
	}	
	
	private JPanel createChooseIDsPanel() {
		JPanel pnl_chooseIDs = new JPanel();
		pnl_chooseIDs.setBorder(BorderFactory.createTitledBorder("Change IDs"));
		pnl_chooseIDs.setLayout(new BoxLayout(pnl_chooseIDs, BoxLayout.PAGE_AXIS));
		pnl_chooseIDs.add(new JLabel("From:"));
		pnl_chooseIDs.add(initIDPane());
		pnl_chooseIDs.add(initSourceIDPanel());
		pnl_chooseIDs.add(new JLabel("To:"));
		pnl_chooseIDs.add(createSelectTargetIDComboBox());
		
		return pnl_chooseIDs;
	}

	private JPanel initSourceIDPanel() {
		JPanel pnl_sourceID = new JPanel(new FlowLayout());
		pnl_sourceID.add(createSelectSourceIDComboBox());
		pnl_sourceID.add(createAddIDButton());
		pnl_sourceID.add(createRemoveIDButton());
		return pnl_sourceID;
	}
	
	private JButton createStartButton() {
		JButton btn_start = new JButton("Start");
		btn_start.addActionListener(this);
		btn_start.setActionCommand("start");
		return btn_start;
	}

	private JButton createRemoveIDButton() {
		JButton btn_removeID = new JButton("Remove ID");
		btn_removeID.addActionListener(this);
		btn_removeID.setActionCommand("removeID");
		return btn_removeID;
	}

	private JButton createAddIDButton() {
		JButton btn_addID = new JButton("Add ID");
		btn_addID.addActionListener(this);
		btn_addID.setActionCommand("addID");
		return btn_addID;
	}

	private JPanel createOpenSaveGamePanel() {
		JPanel pnl_openSaveGame = new JPanel(new FlowLayout());
		pnl_openSaveGame.setBorder(BorderFactory.createTitledBorder("Select a savegame"));
		pnl_openSaveGame.add(new JLabel("Available savegames:"));
		pnl_openSaveGame.add(createSelectSaveGameComboBox());
		pnl_openSaveGame.add(createOpenFileButton());
		return pnl_openSaveGame;
	}
	
	private JScrollPane initIDPane() {
		JScrollPane pn_ID = new JScrollPane(initIDTextArea());
		pn_ID.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		return pn_ID;	
	}
	
	private JList initIDTextArea() {
		li_ID = new JList(model);	
		return li_ID;
	}	
	
	private JComboBox createSelectSourceIDComboBox() {
		if (idNames.size() > 0) {
			cb_selectSourceID = new JComboBox(idNames.toArray());
		} else {
			cb_selectSourceID = new JComboBox();
		}	

		CBRenderer renderer = new CBRenderer();
		renderer.maxIndex = idNames.size()-1;
		
		cb_selectSourceID.setEditable(true);
		cb_selectSourceID.setRenderer(renderer);
		
		cb_selectSourceID.addActionListener(new NumberOnlyActionListener(idNames, 0, 32000-1));
		return cb_selectSourceID;
	}	
	
	private JComboBox createSelectTargetIDComboBox() {
		if (idNames.size() > 0) {
			cb_selectTargetID = new JComboBox(idNames.toArray());
		} else {
			cb_selectTargetID = new JComboBox();
		}	

		CBRenderer renderer = new CBRenderer();
		renderer.maxIndex = idNames.size()-1;
		
		cb_selectTargetID.setEditable(true);
		cb_selectTargetID.setRenderer(renderer);
		
		cb_selectTargetID.addActionListener(new NumberOnlyActionListener(idNames, 0, 32000-1));
		return cb_selectTargetID;
	}		
	
	private JComboBox createSelectSaveGameComboBox() {
		String[] names;
		if (saveGames.size() > 0) {
			names = new String[saveGames.size()];
			for (int i = 0; i < saveGames.size(); i++) {
				names[i] = saveGames.get(i).getName();
			}
		} else {
			names = new String[1];
			names[0] = "";
		}

		// Set initial number of saves for rendering
		CBRenderer renderer = new CBRenderer();
		renderer.maxIndex = names.length-1;
		
		cb_selectSaveGame = new JComboBox(names);
		cb_selectSaveGame.setRenderer(renderer);
		cb_selectSaveGame.addActionListener(this);
		return cb_selectSaveGame;
	}

	private JButton createOpenFileButton() {
		JButton btn_openFile = new JButton("Add savegame");
		btn_openFile.addActionListener(this);
		btn_openFile.setActionCommand("openFolder");
		return btn_openFile;
	}
	
	private String getMCSavePath() {
		return System.getenv("appdata")	+ "/.minecraft/saves";
	}
	
	private boolean isValidSaveGame(File f) {
		ArrayList<RegionFile> rf;
		try {
			rf = RegionFileIO.getRegionFiles(f);
		} catch (IOException e) {
			return false;
		}
		return ((rf != null) && (rf.size() > 0));

	}
	
	private ArrayList<String> readFile(File f) throws IOException {
		ArrayList<String> result = new ArrayList<String>();
		BufferedReader br = new BufferedReader(new FileReader(f));
		String line = br.readLine();
		int index = 0;
		while (line != null) {
		/*	if (!line.equals("")) {
				line = " " + line;
			}
			line = index  + line;*/
			try {
				if (line.contains(" ")) {
					Integer.parseInt(line.substring(0, line.indexOf(" ")));
					result.add(line);
				}
			} catch(NumberFormatException e) {
			}
			line = br.readLine();
			index++;
		}
		br.close();
		return result;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		// something changed - reset progress bars
		pb_file.setValue(0);
		pb_chunk.setValue(0);
		
		// Open Save Folder
		if ("openFolder".equals(e.getActionCommand())) {
			final JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fc.setSelectedFile(new File(getMCSavePath() + "/New World"));
			int returnVal = fc.showOpenDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File f = fc.getSelectedFile();
				if (isValidSaveGame(f)) {
					if (!saveGames.contains(f)) {
						saveGames.add(f);
												
						cb_selectSaveGame.addItem(f.getName());
						cb_selectSaveGame.setSelectedIndex(cb_selectSaveGame.getItemCount()-1);
						
						// workaround for a bug that occurs when there was no item in the beginning
						if (cb_selectSaveGame.getItemCount() > saveGames.size()) {
							cb_selectSaveGame.removeItemAt(0);
						}
						
					//	System.out.println(saveGames.size());
					//	System.out.println(cb_selectSaveGame.getItemCount());
						
					} else {
						cb_selectSaveGame.setSelectedIndex(saveGames.indexOf(f));
						JOptionPane.showMessageDialog(this, "The selected savegame is already in the list!", "Information", JOptionPane.INFORMATION_MESSAGE);
					}
				} else {
					JOptionPane.showMessageDialog(this, "Invalid savegame!", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		// Add ID
		if ("addID".equals(e.getActionCommand())) {		
			if (!model.contains((String)cb_selectSourceID.getSelectedItem())) {
				int index = 0;
				if (model.getSize() > 0) {
					index = model.getSize();
				}
				model.add(index, (String)cb_selectSourceID.getSelectedItem());
			}
		}
			
		// Remove ID
		if ("removeID".equals(e.getActionCommand())) {		
			for (int i=li_ID.getSelectedIndices().length-1; i >= 0; i--) {
				model.remove(li_ID.getSelectedIndices()[i]);	
			}
		//	model.remove(model.indexOf((String)li_ID.getSelectedValue()));
		}		
		// Start
		if ("start".equals(e.getActionCommand())) {		
			// Savegame
			int saveIndex = cb_selectSaveGame.getSelectedIndex();	
		//	System.out.println("Selected Savegame " + saveGames.get(saveIndex));
			try {
				if ((saveGames == null) || (saveGames.size() == 0)) {
					return;
				}
				
				if (saveIndex < 0) {
					return;
				}
				
				final ArrayList<RegionFile> regionFiles = RegionFileIO.getRegionFiles(saveGames.get(saveIndex));
				
				// Backup savegame
				if (c_backup.isSelected()) {
					backUpSaveGame(saveGames.get(saveIndex));
				}

				if (model.size() == 0) {
					JOptionPane.showMessageDialog(this, "No IDs have been chosen!", "Warning", JOptionPane.WARNING_MESSAGE);
					return;
				}
				
				// Source IDs
				final Set<Integer> sourceIDs = new HashSet<Integer>();
				for (int i = 0; i < model.size(); i++) {
					String current = (String) model.get(i);
					
					if (current == null) {
						return;
					}
					
					if (current.contains(" ")) {
						current = current.substring(0, current.indexOf(" "));
					}
					sourceIDs.add(Integer.parseInt(current));
				}				
			
				// Target ID
				String current = (String) cb_selectTargetID.getSelectedItem();
				
				if (current == null) {
					return;
				}				
				
				if (current.contains(" ")) {
					current = current.substring(0, current.indexOf(" "));
				}
				final short targetID = (short)Integer.parseInt(current);

				// change block ids
				SwingWorker worker = new SwingWorker() {
					@Override
					protected Object doInBackground() throws Exception {
						changeIDs(regionFiles, sourceIDs, targetID);
						return null;
					}	
				};
				//worker.addPropertyChangeListener(this);
				worker.execute();
			} catch (IOException e1) {
			}
		}		
	}
	
	private void backUpSaveGame(File file) {
		File regionDir = new File(file, "region");
		if (!regionDir.exists()) {
			return;
		}
		int num = 1;
		File backUpDir = new File(file, "region_backup");
		while (backUpDir.exists()) {
			backUpDir = new File(file, "region_backup_" + num);
			num++;
		}
		try {
			FileTools.copyDirectory(regionDir, backUpDir);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(this, "Could not create backup!", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Changes IDs in the given region files
	 * 
	 * @param regionFiles
	 * @param fromIDs
	 * @param toID
	 * @throws IOException
	 */
	public void changeIDs(ArrayList<RegionFile> regionFiles, Set<Integer> fromIDs, short toID) throws IOException {
		if (regionFiles == null) {
			// No valid region files found
			return;
		}
		
		long beginTime = System.currentTimeMillis();
		
		// PROGESSBAR FILE
		pb_file.setMaximum(regionFiles.size()-1);
		int count_file = 0;
		
		for (RegionFile rf : regionFiles) {
			// Progess
			pb_file.setValue(count_file++);
			lb_file.setText("Current File: " + rf.getFile().getName());
			
			// System.out.println("Processing file " + rf.getFile());
			ArrayList<Point> chunks = new ArrayList<Point>();

			// Get available chunks
			for (int x = 0; x < 32; x++) {
				for (int z = 0; z < 32; z++) {
					if (rf.hasChunk(x, z)) {
						chunks.add(new Point(x, z));
					}
				}
			}
			
			// PROGESSBAR CHUNK
			pb_chunk.setMaximum(chunks.size()-1);
			int count_chunk = 0;

			for (Point p : chunks) {
				// Progress
				pb_chunk.setValue(count_chunk++);
				lb_chunk.setText("Current Chunk: (" + p.x + "; " + p.y + ")");
				
				// Read chunks
				DataInputStream input = rf.getChunkDataInputStream(p.x, p.y);
				Tag root = (new TagInputStream(input)).readTag(true);
				input.close();

				// Find blocks
				
				// Only allow blocks to be replaced with blocks
				if (toID <= 255) {
					Tag t = root.findChildByName("Blocks", true);
					if (t instanceof TagByteArray) {
						TagByteArray blocks = (TagByteArray) t;
						for (int i = 0; i < blocks.payload.length; i++) {
							// Change IDs
							if (fromIDs.contains(new Integer(blocks.payload[i]))) {
								blocks.payload[i] = (byte)toID;
							}
						}
					}
				}
				
				//root.print(0);
				
				// Find items
				ArrayList<Tag> items = new ArrayList<Tag>();
				root.findAllChildrenByName(items, "Items", true);
				for (Tag t : items) {
					if (t instanceof TagList) {
						ArrayList<Tag> ids = new ArrayList<Tag>();
						t.findAllChildrenByName(ids, "id", true);
						for (Tag id : ids) {
							if (id instanceof TagShort) {
								TagShort idShort = (TagShort)id;
								if (fromIDs.contains(new Integer(idShort.payload))) {
									idShort.payload = (short)toID;
								}								
							}
						}
					}
				}

				// Write chunks
				DataOutputStream output = rf.getChunkDataOutputStream(p.x, p.y);
				(new TagOutputStream(output)).writeTag(root, true);
				output.close();
			}
			rf.close();
		}
		long duration = System.currentTimeMillis() - beginTime;
		JOptionPane.showMessageDialog(this, "Done in " + duration + "ms", "Information", JOptionPane.INFORMATION_MESSAGE);
	}	
	
	public static void main(String[] args) {		
		try {
			// Use system specific look and feel
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		}
		IDChanger frame = new IDChanger("mIDas - V0.2");
	}
}
